@@ -323,10 +323,12 @@ void itemize(struct file_struct *file, int ndx, int statret, STRUCT_STAT *st,
 			    : S_ISDIR(file->mode) ? !omit_dir_times
 			    : !S_ISLNK(file->mode);
 
-			if ((iflags & (ITEM_TRANSFER|ITEM_LOCAL_CHANGE) && !keep_time)
-			    || (keep_time && file->modtime != st->st_mtime))
+			if ((iflags & (ITEM_TRANSFER|ITEM_LOCAL_CHANGE) && !keep_time
+			     && (!(iflags & ITEM_XNAME_FOLLOWS) || *xname))
+			    || (keep_time && cmp_modtime(file->modtime, st->st_mtime) != 0))
 				iflags |= ITEM_REPORT_TIME;
-			if (preserve_perms && file->mode != st->st_mode)
+			if (preserve_perms
+			 && (file->mode & CHMOD_BITS) != (st->st_mode & CHMOD_BITS))
 				iflags |= ITEM_REPORT_PERMS;
 			if (preserve_uid && am_root && file->uid != st->st_uid)
 				iflags |= ITEM_REPORT_OWNER;
@@ -528,7 +530,7 @@ static int find_fuzzy(struct file_struct *file, struct file_list *dirlist)
 		name = fp->basename;
 
 		if (fp->length == file->length
-		    && fp->modtime == file->modtime) {
+		    && cmp_modtime(fp->modtime, file->modtime) == 0) {
 			if (verbose > 4) {
 				rprintf(FINFO,
 					"fuzzy size/modtime match for %s\n",
@@ -706,7 +708,7 @@ static void recv_generator(char *fname, struct file_struct *file, int ndx,
 		if (statret != 0 && do_mkdir(fname,file->mode) != 0 && errno != EEXIST) {
 			if (!relative_paths || errno != ENOENT
 			    || create_directory_path(fname, orig_umask) < 0
-			    || do_mkdir(fname, file->mode) < 0) {
+			    || (do_mkdir(fname, file->mode) < 0 && errno != EEXIST)) {
 				rsyserr(FERROR, errno,
 					"recv_generator: mkdir %s failed",
 					full_fname(fname));
