@@ -24,18 +24,20 @@
 
 extern int verbose;
 extern int dry_run;
+extern int daemon_log_format_has_i;
 extern int preserve_times;
+extern int omit_dir_times;
 extern int am_root;
+extern int am_server;
 extern int am_sender;
 extern int am_generator;
+extern int am_starting_up;
 extern int preserve_uid;
 extern int preserve_gid;
-extern int force_delete;
-extern int recurse;
+extern int inplace;
 extern int keep_dirlinks;
 extern int make_backups;
-extern char *backup_dir;
-extern int inplace;
+extern struct stats stats;
 
 
 /*
@@ -48,81 +50,6 @@ void free_sums(struct sum_struct *s)
 }
 
 
-/*
- * delete a file or directory. If force_delete is set then delete
- * recursively
- */
-int delete_file(char *fname)
-{
-	DIR *d;
-	struct dirent *di;
-	char buf[MAXPATHLEN];
-	STRUCT_STAT st;
-	int ret;
-
-#if SUPPORT_LINKS
-	ret = do_lstat(fname, &st);
-#else
-	ret = do_stat(fname, &st);
-#endif
-	if (ret)
-		return -1;
-
-	if (!S_ISDIR(st.st_mode)) {
-		if (robust_unlink(fname) == 0 || errno == ENOENT)
-			return 0;
-		rsyserr(FERROR, errno, "delete_file: unlink %s failed",
-			full_fname(fname));
-		return -1;
-	}
-
-	if (do_rmdir(fname) == 0 || errno == ENOENT)
-		return 0;
-	if (!force_delete || !recurse
-	    || (errno != ENOTEMPTY && errno != EEXIST)) {
-		rsyserr(FERROR, errno, "delete_file: rmdir %s failed",
-			full_fname(fname));
-		return -1;
-	}
-
-	/* now we do a recsursive delete on the directory ... */
-	if (!(d = opendir(fname))) {
-		rsyserr(FERROR, errno, "delete_file: opendir %s failed",
-			full_fname(fname));
-		return -1;
-	}
-
-	for (errno = 0, di = readdir(d); di; errno = 0, di = readdir(d)) {
-		char *dname = d_name(di);
-		if (dname[0] == '.' && (dname[1] == '\0'
-		    || (dname[1] == '.' && dname[2] == '\0')))
-			continue;
-		pathjoin(buf, sizeof buf, fname, dname);
-		if (verbose > 0)
-			rprintf(FINFO, "deleting %s\n", safe_fname(buf));
-		if (delete_file(buf) != 0) {
-			closedir(d);
-			return -1;
-		}
-	}
-	if (errno) {
-		rsyserr(FERROR, errno, "delete_file: readdir %s failed",
-			full_fname(fname));
-		closedir(d);
-		return -1;
-	}
-
-	closedir(d);
-
-	if (do_rmdir(fname) != 0) {
-		rsyserr(FERROR, errno, "delete_file: rmdir %s failed",
-			full_fname(fname));
-		return -1;
-	}
-
-	return 0;
-}
-
 int set_perms(char *fname,struct file_struct *file,STRUCT_STAT *st,
 	      int flags)
 {
@@ -130,10 +57,9 @@ int set_perms(char *fname,struct file_struct *file,STRUCT_STAT *st,
 	STRUCT_STAT st2;
 	int change_uid, change_gid;
 
-	if (dry_run)
-		return 0;
-
 	if (!st) {
+		if (dry_run)
+			return 1;
 		if (link_stat(fname, &st2, 0) < 0) {
 			rsyserr(FERROR, errno, "stat %s failed",
 				full_fname(fname));
@@ -143,14 +69,11 @@ int set_perms(char *fname,struct file_struct *file,STRUCT_STAT *st,
 	}
 
 	if (!preserve_times || S_ISLNK(st->st_mode)
-	    || (make_backups && !backup_dir && S_ISDIR(st->st_mode)))
+	 || (S_ISDIR(st->st_mode) && omit_dir_times))
 		flags |= PERMS_SKIP_MTIME;
 	if (!(flags & PERMS_SKIP_MTIME)
 	    && cmp_modtime(st->st_mtime, file->modtime) != 0) {
-		/* don't complain about not setting times on directories
-		 * because some filesystems can't do it */
-		if (set_modtime(fname,file->modtime) != 0 &&
-		    !S_ISDIR(st->st_mode)) {
+		if (set_modtime(fname,file->modtime) != 0) {
 			rsyserr(FERROR, errno, "failed to set times on %s",
 				full_fname(fname));
 			return 0;
@@ -161,17 +84,24 @@ int set_perms(char *fname,struct file_struct *file,STRUCT_STAT *st,
 	change_uid = am_root && preserve_uid && st->st_uid != file->uid;
 	change_gid = preserve_gid && file->gid != GID_NONE
 		&& st->st_gid != file->gid;
+#if !defined HAVE_LCHOWN && !defined CHOWN_MODIFIES_SYMLINK
+	if (S_ISLNK(st->st_mode))
+		;
+	else
+#endif
 	if (change_uid || change_gid) {
 		if (verbose > 2) {
 			if (change_uid) {
 				rprintf(FINFO,
-				    "set uid of %s from %ld to %ld\n",
-				    fname, (long)st->st_uid, (long)file->uid);
+					"set uid of %s from %ld to %ld\n",
+					safe_fname(fname),
+					(long)st->st_uid, (long)file->uid);
 			}
 			if (change_gid) {
 				rprintf(FINFO,
-				    "set gid of %s from %ld to %ld\n",
-				    fname, (long)st->st_gid, (long)file->gid);
+					"set gid of %s from %ld to %ld\n",
+					safe_fname(fname),
+					(long)st->st_gid, (long)file->gid);
 			}
 		}
 		if (do_lchown(fname,
@@ -185,8 +115,8 @@ int set_perms(char *fname,struct file_struct *file,STRUCT_STAT *st,
 			return 0;
 		}
 		/* a lchown had been done - we have to re-stat if the
-                 * destination had the setuid or setgid bits set due
-                 * to the side effect of the chown call */
+		 * destination had the setuid or setgid bits set due
+		 * to the side effect of the chown call */
 		if (st->st_mode & (S_ISUID | S_ISGID)) {
 			link_stat(fname, st,
 				  keep_dirlinks && S_ISDIR(st->st_mode));
@@ -208,10 +138,12 @@ int set_perms(char *fname,struct file_struct *file,STRUCT_STAT *st,
 #endif
 
 	if (verbose > 1 && flags & PERMS_REPORT) {
+		enum logcode code = daemon_log_format_has_i || dry_run
+				  ? FCLIENT : FINFO;
 		if (updated)
-			rprintf(FINFO,"%s\n",fname);
+			rprintf(code, "%s\n", safe_fname(fname));
 		else
-			rprintf(FINFO,"%s is uptodate\n",fname);
+			rprintf(code, "%s is uptodate\n", safe_fname(fname));
 	}
 	return updated;
 }
@@ -235,30 +167,32 @@ void sig_int(void)
 /* finish off a file transfer, renaming the file and setting the permissions
    and ownership */
 void finish_transfer(char *fname, char *fnametmp, struct file_struct *file,
-		     int ok_to_set_time)
+		     int ok_to_set_time, int overwriting_basis)
 {
 	int ret;
 
 	if (inplace) {
 		if (verbose > 2)
-			rprintf(FINFO, "finishing %s\n", fname);
+			rprintf(FINFO, "finishing %s\n", safe_fname(fname));
 		goto do_set_perms;
 	}
 
-	if (make_backups && !make_backup(fname))
+	if (make_backups && overwriting_basis && !make_backup(fname))
 		return;
 
 	/* Change permissions before putting the file into place. */
 	set_perms(fnametmp, file, NULL, ok_to_set_time ? 0 : PERMS_SKIP_MTIME);
 
 	/* move tmp file over real file */
-	if (verbose > 2)
-		rprintf(FINFO, "renaming %s to %s\n", fnametmp, fname);
+	if (verbose > 2) {
+		rprintf(FINFO, "renaming %s to %s\n",
+			safe_fname(fnametmp), safe_fname(fname));
+	}
 	ret = robust_rename(fnametmp, fname, file->mode & INITACCESSPERMS);
 	if (ret < 0) {
 		rsyserr(FERROR, errno, "%s %s -> \"%s\"",
 		    ret == -2 ? "copy" : "rename",
-		    full_fname(fnametmp), fname);
+		    full_fname(fnametmp), safe_fname(fname));
 		do_unlink(fnametmp);
 		return;
 	}
@@ -272,5 +206,7 @@ void finish_transfer(char *fname, char *fnametmp, struct file_struct *file,
 
 const char *who_am_i(void)
 {
+	if (am_starting_up)
+		return am_server ? "server" : "client";
 	return am_sender ? "sender" : am_generator ? "generator" : "receiver";
 }
